@@ -22,6 +22,10 @@ DEVICE_GET_BASIC_SETTINGS_KEY = "5702"
 DEVICE_SET_MODE_KEY = "5703"
 DEVICE_SET_EXTENDED_KEY = "570f"
 
+# Plug Mini keys
+PLUG_ON_KEY = "570f50010180"
+PLUG_OFF_KEY = "570f50010100"
+
 # Bot keys
 PRESS_KEY = "570100"
 ON_KEY = "570101"
@@ -56,7 +60,7 @@ def _sb_uuid(comms_type: str = "service") -> UUID | str:
     return "Incorrect type, choose between: tx, rx or service"
 
 
-def _process_wohand(data: bytes) -> dict[str, bool | int]:
+def _process_wohand(data: bytes, mfr_data: bytes | None) -> dict[str, bool | int]:
     """Process woHand/Bot services data."""
     _switch_mode = bool(data[1] & 0b10000000)
 
@@ -69,7 +73,9 @@ def _process_wohand(data: bytes) -> dict[str, bool | int]:
     return _bot_data
 
 
-def _process_wocurtain(data: bytes, reverse: bool = True) -> dict[str, bool | int]:
+def _process_wocurtain(
+    data: bytes, mfr_data: bytes | None, reverse: bool = True
+) -> dict[str, bool | int]:
     """Process woCurtain/Curtain services data."""
 
     _position = max(min(data[3] & 0b01111111, 100), 0)
@@ -86,7 +92,7 @@ def _process_wocurtain(data: bytes, reverse: bool = True) -> dict[str, bool | in
     return _curtain_data
 
 
-def _process_wosensorth(data: bytes) -> dict[str, object]:
+def _process_wosensorth(data: bytes, mfr_data: bytes | None) -> dict[str, object]:
     """Process woSensorTH/Temp sensor services data."""
 
     _temp_sign = 1 if data[4] & 0b10000000 else -1
@@ -104,7 +110,7 @@ def _process_wosensorth(data: bytes) -> dict[str, object]:
     return _wosensorth_data
 
 
-def _process_wocontact(data: bytes) -> dict[str, bool | int]:
+def _process_wocontact(data: bytes, mfr_data: bytes | None) -> dict[str, bool | int]:
     """Process woContact Sensor services data."""
     return {
         "tested": bool(data[1] & 0b10000000),
@@ -117,6 +123,29 @@ def _process_wocontact(data: bytes) -> dict[str, bool | int]:
     }
 
 
+def _process_wopresence(data: bytes, mfr_data: bytes | None) -> dict[str, bool | int]:
+    """Process WoPresence Sensor services data."""
+    return {
+        "tested": bool(data[1] & 0b10000000),
+        "motion_detected": bool(data[1] & 0b01000000),
+        "battery": data[2] & 0b01111111,
+        "led": (data[5] & 0b00100000) >> 5,
+        "iot": (data[5] & 0b00010000) >> 4,
+        "sense_distance": (data[5] & 0b00001100) >> 2,
+        "light_intensity": data[5] & 0b00000011,
+        "is_light": bool(data[5] & 0b00000010),
+    }
+
+
+def _process_woplugmini(data: bytes, mfr_data: bytes | None) -> dict[str, bool | int]:
+    """Process plug mini."""
+    return {
+        "switchMode": True,
+        "isOn": mfr_data[7] == 0x80,
+        "wifi_rssi": -mfr_data[9],
+    }
+
+
 @dataclass
 class SwitchBotAdvertisement:
     """Switchbot advertisement."""
@@ -131,17 +160,22 @@ def parse_advertisement_data(
 ) -> SwitchBotAdvertisement | None:
     """Parse advertisement data."""
     _services = list(advertisement_data.service_data.values())
+    _mgr_datas = list(advertisement_data.manufacturer_data.values())
+
     if not _services:
         return
     _service_data = _services[0]
+    _mfr_data = _mgr_datas[0] if _mgr_datas else None
     _model = chr(_service_data[0] & 0b01111111)
 
     supported_types: dict[str, dict[str, Any]] = {
         "d": {"modelName": "WoContact", "func": _process_wocontact},
         "H": {"modelName": "WoHand", "func": _process_wohand},
+        "s": {"modelName": "WoPresence", "func": _process_wopresence},
         "c": {"modelName": "WoCurtain", "func": _process_wocurtain},
         "T": {"modelName": "WoSensorTH", "func": _process_wosensorth},
         "i": {"modelName": "WoSensorTH", "func": _process_wosensorth},
+        "g": {"modelName": "WoPlug", "func": _process_woplugmini},
     }
 
     data = {
@@ -159,7 +193,7 @@ def parse_advertisement_data(
                 "isEncrypted": bool(_service_data[0] & 0b10000000),
                 "model": _model,
                 "modelName": supported_types[_model]["modelName"],
-                "data": supported_types[_model]["func"](_service_data),
+                "data": supported_types[_model]["func"](_service_data, _mfr_data),
             }
         )
 
@@ -290,11 +324,6 @@ class SwitchbotDevice:
             self._password_encoded = "%x" % (
                 binascii.crc32(password.encode("ascii")) & 0xFFFFFFFF
             )
-        self._last_notification = bytearray()
-
-    async def _notification_handler(self, sender: int, data: bytearray) -> None:
-        """Handle notification responses."""
-        self._last_notification = data
 
     def _commandkey(self, key: str) -> str:
         """Add password to key if set."""
@@ -325,33 +354,41 @@ class SwitchbotDevice:
 
         raise RuntimeError("Unreachable")
 
+    @property
+    def name(self) -> str:
+        """Return device name."""
+        return f"{self._device.name} ({self._device.address})"
+
     async def _send_command_locked(self, key: str, command: bytes) -> bytes:
         """Send command to device and read response."""
         client: BleakClient | None = None
         try:
-            _LOGGER.debug("Connnecting to switchbot: %s", self._device.address)
-
+            _LOGGER.debug("%s: Connnecting to switchbot", self.name)
             client = await establish_connection(
-                BleakClient, self._device.address, self._device, max_attempts=1
+                BleakClient, self._device, self.name, max_attempts=1
             )
-            _LOGGER.debug("Connnected to switchbot: %s", client.is_connected)
-
-            _LOGGER.debug("Subscribe to notifications")
-            await client.start_notify(
-                _sb_uuid(comms_type="rx"), self._notification_handler
+            _LOGGER.debug(
+                "%s: Connnected to switchbot: %s", self.name, client.is_connected
             )
+            future: asyncio.Future[bytearray] = asyncio.Future()
 
-            _LOGGER.debug("Sending command, %s", key)
+            def _notification_handler(sender: int, data: bytearray) -> None:
+                """Handle notification responses."""
+                if future.done():
+                    _LOGGER.debug("%s: Notification handler already done", self.name)
+                    return
+                future.set_result(data)
+
+            _LOGGER.debug("%s: Subscribe to notifications", self.name)
+            await client.start_notify(_sb_uuid(comms_type="rx"), _notification_handler)
+
+            _LOGGER.debug("%s: Sending command, %s", self.name, key)
             await client.write_gatt_char(_sb_uuid(comms_type="tx"), command, False)
 
-            await asyncio.sleep(
-                1.0
-            )  # Bot needs pause. Otherwise notification could be missed.
+            notify_msg = await asyncio.wait_for(future, timeout=5)
+            _LOGGER.info("%s: Notification received: %s", self.name, notify_msg)
 
-            notify_msg = self._last_notification
-            _LOGGER.info("Notification received: %s", notify_msg)
-
-            _LOGGER.debug("UnSubscribe to notifications")
+            _LOGGER.debug("%s: UnSubscribe to notifications", self.name)
             await client.stop_notify(_sb_uuid(comms_type="rx"))
 
         finally:
@@ -719,3 +756,34 @@ class SwitchbotCurtain(SwitchbotDevice):
         """Return True curtain is calibrated."""
         # To get actual light level call update() first.
         return self._get_adv_value("calibration")
+
+
+class SwitchbotPlugMini(SwitchbotDevice):
+    """Representation of a Switchbot plug mini."""
+
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
+        """Switchbot plug mini constructor."""
+        super().__init__(*args, **kwargs)
+        self._settings: dict[str, Any] = {}
+
+    async def update(self, interface: int | None = None) -> None:
+        """Update state of device."""
+        await self.get_device_data(retry=self._retry_count, interface=interface)
+
+    async def turn_on(self) -> bool:
+        """Turn device on."""
+        result = await self._sendcommand(PLUG_ON_KEY, self._retry_count)
+        return result[1] == 0x80
+
+    async def turn_off(self) -> bool:
+        """Turn device off."""
+        result = await self._sendcommand(PLUG_OFF_KEY, self._retry_count)
+        return result[1] == 0x00
+
+    def is_on(self) -> Any:
+        """Return switch state from cache."""
+        # To get actual position call update() first.
+        value = self._get_adv_value("isOn")
+        if value is None:
+            return None
+        return value
